@@ -1,19 +1,9 @@
 /*
- * Copyright (C) 2011-2017 Red Hat, Inc.
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * Copyright (C) 2011-2023 Red Hat, Inc.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; If not, see <http://www.gnu.org/licenses/>.
- *
- * Author: tasleson
+ * Author: Tony Asleson <tasleson@redhat.com>
  */
 
 #ifndef LIBSTORAGEMGMTERROR_H
@@ -91,6 +81,9 @@ typedef enum {
     /** NFS export not found */
     LSM_ERR_NOT_FOUND_NFS_EXPORT = 206,
 
+    /** Some resource not found, non-specific, added in 1.10 */
+    LSM_ERR_NOT_FOUND_GENERIC = 207,
+
     /** System not found */
     LSM_ERR_NOT_FOUND_SYSTEM = 208,
     /** Disk not found */
@@ -119,7 +112,7 @@ typedef enum {
     /** Insufficient space */
     LSM_ERR_NOT_ENOUGH_SPACE = 350,
 
-    /** Error comunicating with plug-in */
+    /** Error communicating with plug-in */
     LSM_ERR_TRANSPORT_COMMUNICATION = 400,
     /** Transport serialization error */
     LSM_ERR_TRANSPORT_SERIALIZATION = 401,
@@ -128,7 +121,7 @@ typedef enum {
 
     LSM_ERR_LAST_INIT_IN_ACCESS_GROUP = 502,
 
-    /** Unsupport search key */
+    /** Unsupported search key */
     LSM_ERR_UNSUPPORTED_SEARCH_KEY = 510,
 
     LSM_ERR_EMPTY_ACCESS_GROUP = 511,
