@@ -1,18 +1,8 @@
 /*
- * Copyright (C) 2016-2017 Red Hat, Inc.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
+ * Copyright (C) 2016-2023 Red Hat, Inc.
  * (C) Copyright (C) 2017 Hewlett Packard Enterprise Development LP
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; If not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Gris Ge <fge@redhat.com>
  */
@@ -152,8 +142,8 @@ struct _sg_t10_vpd83_header {
     uint8_t dev_type  : 5;
     uint8_t qualifier : 3;
 #else
-    uint8_t qualifier      : 3;
-    uint8_t dev_type       : 5;
+    uint8_t qualifier : 3;
+    uint8_t dev_type  : 5;
 #endif
     uint8_t page_code;
     uint16_t page_len_be;
@@ -164,8 +154,8 @@ struct _sg_t10_vpd80_header {
     uint8_t dev_type  : 5;
     uint8_t qualifier : 3;
 #else
-    uint8_t qualifier      : 3;
-    uint8_t dev_type       : 5;
+    uint8_t qualifier : 3;
+    uint8_t dev_type  : 5;
 #endif
     uint8_t page_code;
     uint16_t page_len_be;
@@ -196,8 +186,8 @@ struct _sg_t10_sense_fixed {
     uint8_t response_code : 7;
     uint8_t valid         : 1;
 #else
-    uint8_t valid          : 1;
-    uint8_t response_code  : 7;
+    uint8_t valid         : 1;
+    uint8_t response_code : 7;
 #endif
     uint8_t reserved;
 #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
@@ -254,8 +244,8 @@ struct _sg_t10_info_excep_mode_page_0_hdr {
     uint8_t reserved : 4;
     uint8_t mrie     : 4;
 #else
-    uint8_t mrie           : 4;
-    uint8_t reserved       : 4;
+    uint8_t mrie     : 4;
+    uint8_t reserved : 4;
 #endif
 };
 
@@ -298,9 +288,9 @@ struct _sg_t10_ata_pass_through_12_cdb {
     uint8_t ck_cond    : 1;
     uint8_t off_line   : 2;
 #else
-    uint8_t obsolete       : 3;
-    uint8_t protocol       : 4;
-    uint8_t reserved_0     : 1;
+    uint8_t obsolete   : 3;
+    uint8_t protocol   : 4;
+    uint8_t reserved_0 : 1;
 
     uint8_t off_line   : 2;
     uint8_t ck_cond    : 1;
@@ -745,7 +735,6 @@ int _sg_parse_vpd_83(char *err_msg, uint8_t *vpd_data,
 
         (*dps)[*dp_count] = dp;
         ++*dp_count;
-        dp = NULL;
     }
 
 out:
