@@ -1,19 +1,9 @@
 /*
- * Copyright (C) 2011-2014 Red Hat, Inc.
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * Copyright (C) 2011-2023 Red Hat, Inc.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; If not, see <http://www.gnu.org/licenses/>.
- *
- * Author: tasleson
+ * Author: Tony Asleson <tasleson@redhat.com>
  */
 
 #include "libstoragemgmt/libstoragemgmt.h"
@@ -171,13 +161,6 @@ static int verify_initiator_id(const char *id, lsm_access_group_init_type t,
  */
 #define CHECK_STR(x) (!(x) || !strlen(x))
 
-/**
- * When we pass in a pointer for an out value we want to make sure that
- * the pointer isn't null, and that the dereferenced value is != NULL to prevent
- * memory leaks.
- */
-#define CHECK_RP(x) (!(x) || *(x) != NULL)
-
 int lsm_connect_password(const char *uri, const char *password,
                          lsm_connect **conn, uint32_t timeout, lsm_error_ptr *e,
                          lsm_flag flags) {
@@ -380,16 +363,17 @@ int lsm_plugin_info_get(lsm_connect *c, char **desc, char **version,
 
             if (!*desc || !*version) {
                 rc = LSM_ERR_NO_MEMORY;
-                free(*desc);
-                free(*version);
             }
         }
     } catch (const ValueException &ve) {
+        rc = log_exception(c, LSM_ERR_PLUGIN_BUG, "Unexpected type", ve.what());
+    }
+
+    if (rc != LSM_ERR_OK) {
         free(*desc);
         *desc = NULL;
         free(*version);
         *version = NULL;
-        rc = log_exception(c, LSM_ERR_PLUGIN_BUG, "Unexpected type", ve.what());
     }
 
     return rc;
@@ -1081,8 +1065,8 @@ int lsm_volume_create(lsm_connect *c, lsm_pool *pool, const char *volumeName,
 
     int rc = rpc(c, "volume_create", parameters, response);
     if (LSM_ERR_OK == rc) {
-        *newVolume = (lsm_volume *)parse_job_response(c, response, rc, job,
-                                                      (convert)value_to_volume);
+        *newVolume = (lsm_volume *)parse_job_response(
+            c, std::move(response), rc, job, (convert)value_to_volume);
     }
     return rc;
 }
@@ -1111,7 +1095,7 @@ int lsm_volume_resize(lsm_connect *c, lsm_volume *volume, uint64_t newSize,
     int rc = rpc(c, "volume_resize", parameters, response);
     if (LSM_ERR_OK == rc) {
         *resizedVolume = (lsm_volume *)parse_job_response(
-            c, response, rc, job, (convert)value_to_volume);
+            c, std::move(response), rc, job, (convert)value_to_volume);
     }
     return rc;
 }
@@ -1144,7 +1128,7 @@ int lsm_volume_replicate(lsm_connect *c, lsm_pool *pool,
     int rc = rpc(c, "volume_replicate", parameters, response);
     if (LSM_ERR_OK == rc) {
         *newReplicant = (lsm_volume *)parse_job_response(
-            c, response, rc, job, (convert)value_to_volume);
+            c, std::move(response), rc, job, (convert)value_to_volume);
     }
     return rc;
 }
@@ -1380,7 +1364,7 @@ int lsm_access_group_create(lsm_connect *c, const char *name,
 
     std::map<std::string, Value> p;
     p["name"] = Value(name);
-    p["init_id"] = id;
+    p["init_id"] = std::move(id);
     p["init_type"] = Value((int32_t)init_type);
     p["system"] = system_to_value(system);
     p["flags"] = Value(flags);
@@ -1445,7 +1429,7 @@ static int _lsm_ag_add_delete(lsm_connect *c, lsm_access_group *access_group,
 
     std::map<std::string, Value> p;
     p["access_group"] = access_group_to_value(access_group);
-    p["init_id"] = id;
+    p["init_id"] = std::move(id);
     p["init_type"] = Value((int32_t)init_type);
     p["flags"] = Value(flags);
 
@@ -1813,7 +1797,7 @@ int lsm_fs_create(lsm_connect *c, lsm_pool *pool, const char *name,
 
     int rc = rpc(c, "fs_create", parameters, response);
     if (LSM_ERR_OK == rc) {
-        *fs = (lsm_fs *)parse_job_response(c, response, rc, job,
+        *fs = (lsm_fs *)parse_job_response(c, std::move(response), rc, job,
                                            (convert)value_to_fs);
     }
     return rc;
@@ -1859,7 +1843,7 @@ int lsm_fs_resize(lsm_connect *c, lsm_fs *fs, uint64_t new_size_bytes,
 
     int rc = rpc(c, "fs_resize", parameters, response);
     if (LSM_ERR_OK == rc) {
-        *rfs = (lsm_fs *)parse_job_response(c, response, rc, job,
+        *rfs = (lsm_fs *)parse_job_response(c, std::move(response), rc, job,
                                             (convert)value_to_fs);
     }
     return rc;
@@ -1886,8 +1870,8 @@ int lsm_fs_clone(lsm_connect *c, lsm_fs *src_fs, const char *name,
 
     int rc = rpc(c, "fs_clone", parameters, response);
     if (LSM_ERR_OK == rc) {
-        *cloned_fs = (lsm_fs *)parse_job_response(c, response, rc, job,
-                                                  (convert)value_to_fs);
+        *cloned_fs = (lsm_fs *)parse_job_response(c, std::move(response), rc,
+                                                  job, (convert)value_to_fs);
     }
     return rc;
 }
@@ -2063,8 +2047,8 @@ int lsm_fs_ss_create(lsm_connect *c, lsm_fs *fs, const char *name,
 
     int rc = rpc(c, "fs_snapshot_create", parameters, response);
     if (LSM_ERR_OK == rc) {
-        *snapshot = (lsm_fs_ss *)parse_job_response(c, response, rc, job,
-                                                    (convert)value_to_ss);
+        *snapshot = (lsm_fs_ss *)parse_job_response(c, std::move(response), rc,
+                                                    job, (convert)value_to_ss);
     }
     return rc;
 }
