@@ -1,75 +1,62 @@
 /*
- * Copyright (C) 2011-2015 Red Hat, Inc.
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * Copyright (C) 2011-2023 Red Hat, Inc.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; If not, see <http://www.gnu.org/licenses/>.
- *
- * Author: tasleson
+ * Author: Tony Asleson <tasleson@redhat.com>
  */
 
 #define _GNU_SOURCE
+#include <assert.h>
+#include <ctype.h>
+#include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <getopt.h>
+#include <grp.h>
+#include <libconfig.h>
+#include <libgen.h>
+#include <limits.h>
+#include <pwd.h>
+#include <signal.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <getopt.h>
 #include <string.h>
-#include <ctype.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <pwd.h>
-#include <errno.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <syslog.h>
-#include <stdarg.h>
-#include <dirent.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <sys/un.h>
+#include <sys/file.h>
 #include <sys/queue.h>
-#include <sys/wait.h>
+#include <sys/socket.h>
+#include <sys/stat.h>
 #include <sys/time.h>
-#include <libgen.h>
-#include <assert.h>
-#include <grp.h>
-#include <limits.h>
-#include <libconfig.h>
+#include <sys/types.h>
+#include <sys/un.h>
+#include <sys/wait.h>
+#include <syslog.h>
+#include <unistd.h>
 
-#define BASE_DIR  "/var/run/lsm"
-#define SOCKET_DIR BASE_DIR"/ipc"
-#define PLUGIN_DIR "/usr/bin"
-#define LSM_USER "libstoragemgmt"
-#define LSM_CONF_DIR "/etc/lsm/"
-#define LSM_PLUGIN_CONF_DIR_NAME "pluginconf.d"
-#define LSMD_CONF_FILE "lsmd.conf"
-#define LSM_CONF_ALLOW_ROOT_OPT_NAME "allow-plugin-root-privilege"
+#define BASE_DIR                       "/var/run/lsm"
+#define SOCKET_DIR                     BASE_DIR "/ipc"
+#define PLUGIN_DIR                     "/usr/bin"
+#define LSM_USER                       "libstoragemgmt"
+#define LSM_CONF_DIR                   "/etc/lsm/"
+#define LSM_PLUGIN_CONF_DIR_NAME       "pluginconf.d"
+#define LSMD_CONF_FILE                 "lsmd.conf"
+#define LSM_CONF_ALLOW_ROOT_OPT_NAME   "allow-plugin-root-privilege"
 #define LSM_CONF_REQUIRE_ROOT_OPT_NAME "require-root-privilege"
 
-#define min(a,b) \
-   ({ __typeof__ (a) _a = (a); \
-       __typeof__ (b) _b = (b); \
-     _a < _b ? _a : _b; })
-
-#define max(a,b) \
-   ({ __typeof__ (a) _a = (a); \
-       __typeof__ (b) _b = (b); \
-     _a > _b ? _a : _b; })
+#define max(a, b)                                                              \
+    ({                                                                         \
+        __typeof__(a) _a = (a);                                                \
+        __typeof__(b) _b = (b);                                                \
+        _a > _b ? _a : _b;                                                     \
+    })
 
 int verbose_flag = 0;
 int systemd = 0;
 
-char *socket_dir = SOCKET_DIR;
-char *plugin_dir = PLUGIN_DIR;
-char *conf_dir = LSM_CONF_DIR;
+const char *socket_dir = SOCKET_DIR;
+const char *plugin_dir = PLUGIN_DIR;
+const char *conf_dir = LSM_CONF_DIR;
 
 char plugin_extension[] = "_lsmplugin";
 
@@ -104,8 +91,7 @@ LIST_HEAD(plugin_list, plugin) head;
  * @param fmt           String with format
  * @param ...           Format parameters
  */
-void logger(int severity, const char *fmt, ...)
-{
+void logger(int severity, const char *fmt, ...) {
     char buf[2048];
 
     if (verbose_flag || LOG_WARNING == severity || LOG_ERR == severity) {
@@ -131,16 +117,15 @@ void logger(int severity, const char *fmt, ...)
     }
 }
 
-#define log_and_exit(fmt, ...)  logger(LOG_ERR, fmt, ##__VA_ARGS__)
-#define warn(fmt, ...)  logger(LOG_WARNING, fmt, ##__VA_ARGS__)
-#define info(fmt, ...)  logger(LOG_INFO, fmt, ##__VA_ARGS__)
+#define log_and_exit(fmt, ...) logger(LOG_ERR, fmt, ##__VA_ARGS__)
+#define warn(fmt, ...)         logger(LOG_WARNING, fmt, ##__VA_ARGS__)
+#define info(fmt, ...)         logger(LOG_INFO, fmt, ##__VA_ARGS__)
 
 /**
  * Our signal handler.
  * @param s     Received signal
  */
-void signal_handler(int s)
-{
+void signal_handler(int s) {
     if (SIGTERM == s) {
         serve_state = EXIT;
     } else if (SIGHUP == s) {
@@ -151,8 +136,7 @@ void signal_handler(int s)
 /**
  * Installs our signal handler
  */
-void install_sh(void)
-{
+void install_sh(void) {
     if (signal(SIGTERM, signal_handler) == SIG_ERR) {
         log_and_exit("Can't catch signal SIGTERM\n");
     }
@@ -166,8 +150,7 @@ void install_sh(void)
  * If we are running as root, we will try to drop our privs. to our default
  * user.
  */
-void drop_privileges(void)
-{
+void drop_privileges(void) {
     int err = 0;
     struct passwd *pw = NULL;
 
@@ -200,8 +183,7 @@ void drop_privileges(void)
 /**
  * Check to make sure we have access to the directories of interest
  */
-void flight_check(void)
-{
+void flight_check(void) {
     int err = 0;
     if (-1 == access(socket_dir, R_OK | W_OK)) {
         err = errno;
@@ -219,17 +201,17 @@ void flight_check(void)
 /**
  * Print help.
  */
-void usage(void)
-{
+void usage(void) {
     printf("libStorageMgmt plug-in daemon.\n");
     printf("lsmd [--plugindir <directory>] [--socketdir <dir>] [-v] [-d]\n");
     printf("     --plugindir = The directory where the plugins are located\n");
-    printf("     --socketdir = The directory where the Unix domain sockets will "
-                                "be created\n");
+    printf(
+        "     --socketdir = The directory where the Unix domain sockets will "
+        "be created\n");
     printf("     --confdir   = The directory where the config files are "
-                               "located\n");
+           "located\n");
     printf("     -v          = Verbose logging\n");
-    printf("     -d          = new style daemon (systemd)\n");
+    printf("     -d          = New style daemon (systemd)\n");
 }
 
 /**
@@ -238,20 +220,19 @@ void usage(void)
  * @param name      File name
  * @return Concatenated string, caller must call free when done
  */
-char *path_form(const char *path, const char *name)
-{
+char *path_form(const char *path, const char *name) {
     size_t s = strlen(path) + strlen(name) + 2;
     char *full = calloc(1, s);
     if (full) {
         snprintf(full, s, "%s/%s", path, name);
     } else {
-        log_and_exit("malloc failure while trying to allocate %d bytes\n", s);
+        log_and_exit("malloc failure while trying to allocate %zu bytes\n", s);
     }
     return full;
 }
 
 /* Call back signature */
-typedef int (*file_op) (void *p, char *full_file_path);
+typedef int (*file_op)(void *p, char *full_file_path);
 
 /**
  * For a given directory iterate through each directory item and exec the
@@ -261,8 +242,7 @@ typedef int (*file_op) (void *p, char *full_file_path);
  * @param   call_back   Function to call against file
  * @return
  */
-void process_directory(char *dir, void *p, file_op call_back)
-{
+void process_directory(const char *dir, void *p, file_op call_back) {
     int err = 0;
 
     if (call_back && dir && strlen(dir)) {
@@ -314,52 +294,65 @@ void process_directory(char *dir, void *p, file_op call_back)
  * @param full_name     Full path an and file name
  * @return 0 to continue processing, anything else to stop.
  */
-int delete_socket(void *p, char *full_name)
-{
+int delete_socket(void *p, char *full_name) {
     struct stat statbuf;
     int err;
+    int fd = 0;
+    char *dir = NULL;
+    char *full_name_dupe = NULL;
 
     assert(p == NULL);
 
-    if (!lstat(full_name, &statbuf)) {
+    /*
+     *  We need to duplicate the full_name as dirname *may* modify it, also it
+     * may segfault if the string is static
+     */
+    full_name_dupe = strdup(full_name);
+    if (!full_name_dupe) {
+        log_and_exit("Memory allocation error when "
+                     "duplicating delete_socket:full_name\n");
+    }
+
+    dir = dirname(full_name_dupe);
+    fd = open(dir, O_DIRECTORY);
+    if (fd == -1) {
+        err = errno;
+        log_and_exit("Error on opening  directory %s, full name %s: %s\n", dir,
+                     full_name, strerror(err));
+    }
+
+    if (!fstatat(fd, full_name, &statbuf, AT_SYMLINK_NOFOLLOW)) {
         if (S_ISSOCK(statbuf.st_mode)) {
-            if (unlink(full_name)) {
+            if (unlinkat(fd, full_name, 0)) {
                 err = errno;
-                log_and_exit("Error on unlinking file %s: %s\n",
-                             full_name, strerror(err));
+                log_and_exit("Error on unlinking file %s: %s\n", full_name,
+                             strerror(err));
             }
         }
     }
+
+    if (-1 == close(fd)) {
+        err = errno;
+        log_and_exit("Error on closing directory %s: %s\n", dir, strerror(err));
+    }
+
+    free(full_name_dupe);
+
     return 0;
 }
 
 /**
  * Walk the IPC socket directory and remove the socket files.
  */
-void clean_sockets()
-{
-    process_directory(socket_dir, NULL, delete_socket);
-}
-
+void clean_sockets(void) { process_directory(socket_dir, NULL, delete_socket); }
 
 /**
- * Given a fully qualified path and name to a plug-in, create the IPC socket.
- * @param full_name     Full name and path for plug-in
+ * Given a socket file name, create the IPC socket.
+ * @param name     socket file name for plug-in
  * @return  listening socket descriptor for IPC
  */
-int setup_socket(char *full_name)
-{
+int setup_socket(char *name) {
     int err = 0;
-    char name[128];
-
-    /* Strip off _lsmplugin from the file name, not sure
-     * why I chose to do this */
-    memset(name, 0, sizeof(name));
-
-    char *base_nm = basename(full_name);
-    strncpy(name, base_nm,
-            min(abs(strlen(base_nm) - strlen(plugin_extension)),
-                (sizeof(name) - 1)));
 
     char *socket_file = path_form(socket_dir, name);
     delete_socket(NULL, socket_file);
@@ -373,14 +366,14 @@ int setup_socket(char *full_name)
         strncpy(addr.sun_path, socket_file, sizeof(addr.sun_path) - 1);
 
         if (-1 ==
-            bind(fd, (struct sockaddr *) &addr, sizeof(struct sockaddr_un))) {
+            bind(fd, (struct sockaddr *)&addr, sizeof(struct sockaddr_un))) {
             err = errno;
             log_and_exit("Error on binding socket %s: %s\n", socket_file,
                          strerror(err));
         }
 
-        if (-1 == chmod(socket_file, S_IREAD | S_IWRITE | S_IRGRP | S_IWGRP
-                        | S_IROTH | S_IWOTH)) {
+        if (-1 == chmod(socket_file, S_IREAD | S_IWRITE | S_IRGRP | S_IWGRP |
+                                         S_IROTH | S_IWOTH)) {
             err = errno;
             log_and_exit("Error on chmod socket file %s: %s\n", socket_file,
                          strerror(err));
@@ -394,8 +387,8 @@ int setup_socket(char *full_name)
 
     } else {
         err = errno;
-        log_and_exit("Error on socket create %s: %s\n",
-                     socket_file, strerror(err));
+        log_and_exit("Error on socket create %s: %s\n", socket_file,
+                     strerror(err));
     }
 
     free(socket_file);
@@ -406,8 +399,7 @@ int setup_socket(char *full_name)
  * Closes all the listening sockets and re-claims memory in linked list.
  * @param list
  */
-void empty_plugin_list(struct plugin_list *list)
-{
+void empty_plugin_list(struct plugin_list *list) {
     int err;
     struct plugin *item = NULL;
 
@@ -439,13 +431,12 @@ void empty_plugin_list(struct plugin_list *list)
  * @param value         int, output, value of this config key
  */
 
-void parse_conf_bool(char *conf_path, char *key_name, int *value)
-{
+void parse_conf_bool(const char *conf_path, const char *key_name, int *value) {
     if (access(conf_path, F_OK) == -1) {
         /* file not exist. */
         return;
     }
-    config_t *cfg = (config_t *) malloc(sizeof(config_t));
+    config_t *cfg = (config_t *)malloc(sizeof(config_t));
     if (cfg) {
         config_init(cfg);
         if (CONFIG_TRUE == config_read_file(cfg, conf_path)) {
@@ -456,8 +447,8 @@ void parse_conf_bool(char *conf_path, char *key_name, int *value)
                          config_error_line(cfg));
         }
     } else {
-        log_and_exit
-            ("malloc failure while trying to allocate memory for config_t\n");
+        log_and_exit(
+            "malloc failure while trying to allocate memory for config_t\n");
     }
 
     config_destroy(cfg);
@@ -467,45 +458,40 @@ void parse_conf_bool(char *conf_path, char *key_name, int *value)
 /**
  * Load plugin config for root privilege setting.
  * If config not found, return 0 for no root privilege required.
- * @param plugin_path Full path of plugin
+ * @param plugin_name plugin name.
  * @return 1 for require root privilege, 0 or not.
  */
 
-int chk_pconf_root_pri(char *plugin_path)
-{
+int chk_pconf_root_pri(char *plugin_name) {
     int require_root = 0;
-    char *base_name = basename(plugin_path);
-    ssize_t plugin_name_len = strlen(base_name) - strlen(plugin_extension);
-    if (plugin_name_len <= 0) {
-        log_and_exit("Got invalid plugin full path %s\n", plugin_path);
-    }
-    ssize_t conf_file_name_len = plugin_name_len +
-        strlen(plugin_conf_extension) + 1;
-    char *plugin_conf_filename = (char *) malloc(conf_file_name_len);
+    size_t plugin_name_len = strlen(plugin_name);
+    size_t conf_ext_len = strlen(plugin_conf_extension);
+    size_t conf_file_name_len = plugin_name_len + conf_ext_len + 1;
+    char *plugin_conf_filename = (char *)malloc(conf_file_name_len);
+
     if (plugin_conf_filename) {
-        strncpy(plugin_conf_filename, base_name, plugin_name_len);
-        strncpy(plugin_conf_filename + plugin_name_len,
-                plugin_conf_extension, strlen(plugin_conf_extension));
-        plugin_conf_filename[conf_file_name_len - 1] = '\0';
+        snprintf(plugin_conf_filename, conf_file_name_len, "%s%s", plugin_name,
+                 plugin_conf_extension);
 
-        char *plugin_conf_dir_path = path_form(conf_dir,
-                                               LSM_PLUGIN_CONF_DIR_NAME);
+        char *plugin_conf_dir_path =
+            path_form(conf_dir, LSM_PLUGIN_CONF_DIR_NAME);
 
-        char *plugin_conf_path = path_form(plugin_conf_dir_path,
-                                           plugin_conf_filename);
+        char *plugin_conf_path =
+            path_form(plugin_conf_dir_path, plugin_conf_filename);
         parse_conf_bool(plugin_conf_path, LSM_CONF_REQUIRE_ROOT_OPT_NAME,
                         &require_root);
 
         if (require_root == 1 && allow_root_plugin == 0) {
             warn("Plugin %s require root privilege while %s disable globally\n",
-                 base_name, LSMD_CONF_FILE);
+                 plugin_name, LSMD_CONF_FILE);
         }
         free(plugin_conf_dir_path);
         free(plugin_conf_filename);
         free(plugin_conf_path);
     } else {
-        log_and_exit("malloc failure while trying to allocate %d "
-                     "bytes\n", conf_file_name_len);
+        log_and_exit("malloc failure while trying to allocate %zu "
+                     "bytes\n",
+                     conf_file_name_len);
     }
     return require_root;
 }
@@ -516,39 +502,60 @@ int chk_pconf_root_pri(char *plugin_path)
  * @param full_name     Full path and file name
  * @return 0 to continue, else abort directory processing
  */
-int process_plugin(void *p, char *full_name)
-{
-    if (full_name) {
-        size_t ext_len = strlen(plugin_extension);
-        size_t full_len = strlen(full_name);
+int process_plugin(void *p, char *full_name) {
+    char *base_nm = NULL;
+    size_t base_nm_len = 0;
+    size_t no_ext_len = 0;
+    char plugin_name[128];
+    size_t ext_len = strlen(plugin_extension);
+    size_t plugin_name_max_len = sizeof(plugin_name) / sizeof(char);
 
-        if (full_len > ext_len) {
-            if (strncmp
-                (full_name + full_len - ext_len, plugin_extension,
-                 ext_len) == 0) {
-                struct plugin *item = calloc(1, sizeof(struct plugin));
-                if (item) {
-                    item->file_path = strdup(full_name);
-                    item->fd = setup_socket(full_name);
-                    item->require_root = chk_pconf_root_pri(full_name);
-                    has_root_plugin |= item->require_root;
+    if (full_name == NULL)
+        return 0;
 
-                    if (item->file_path && item->fd >= 0) {
-                        LIST_INSERT_HEAD((struct plugin_list *) p, item,
-                                         pointers);
-                        info("Plugin %s added\n", full_name);
-                    } else {
-                        /* The only real way to get here is failed strdup as
-                           setup_socket will exit on error. */
-                        free(item);
-                        item = NULL;
-                        log_and_exit("strdup failed %s\n", full_name);
-                    }
-                } else {
-                    log_and_exit("Memory allocation failure!\n");
-                }
-            }
+    base_nm = basename(full_name);
+    base_nm_len = strlen(base_nm);
+
+    if (base_nm_len <= ext_len)
+        return 0;
+
+    if (strncmp(base_nm + base_nm_len - ext_len, plugin_extension, ext_len))
+        return 0;
+
+    struct plugin *item = calloc(1, sizeof(struct plugin));
+    if (item == NULL) {
+        log_and_exit("Memory allocation failure!\n");
+        return 0; // no use, just trick covscan;
+    }
+
+    /* Strip off _lsmplugin from the file name, not sure
+     * why I chose to do this */
+    memset(plugin_name, 0, plugin_name_max_len);
+    strncpy(plugin_name, base_nm, plugin_name_max_len - 1);
+    no_ext_len = base_nm_len - ext_len;
+    // Already check, no_ext_len is bigger than 0 here.
+    if (no_ext_len < plugin_name_max_len - 1)
+        plugin_name[no_ext_len] = '\0';
+
+    item->file_path = strdup(full_name);
+    item->fd = setup_socket(plugin_name);
+    item->require_root = chk_pconf_root_pri(plugin_name);
+    has_root_plugin |= item->require_root;
+
+    if (item->file_path && item->fd >= 0) {
+        LIST_INSERT_HEAD((struct plugin_list *)p, item, pointers);
+        info("Plugin %s added\n", full_name);
+    } else {
+        /* The only real way to get here is failed strdup as
+           setup_socket will exit on error. */
+
+        /* static code checkers incorrectly flag a failure to close fd */
+        if (item->fd >= 0) {
+            close(item->fd);
         }
+        free(item);
+        item = NULL;
+        log_and_exit("strdup failed %s\n", full_name);
     }
     return 0;
 }
@@ -556,8 +563,7 @@ int process_plugin(void *p, char *full_name)
 /**
  * Cleans up any children that have exited.
  */
-void child_cleanup(void)
-{
+void child_cleanup(void) {
     int rc;
     int err;
 
@@ -589,8 +595,7 @@ void child_cleanup(void)
 /**
  * Closes and frees memory and removes Unix domain sockets.
  */
-void clean_up(void)
-{
+void clean_up(void) {
     empty_plugin_list(&head);
     clean_sockets();
 }
@@ -599,8 +604,7 @@ void clean_up(void)
  * Walks the plugin directory creating IPC sockets for each one.
  * @return
  */
-int process_plugins(void)
-{
+int process_plugins(void) {
     clean_up();
     info("Scanning plug-in directory %s\n", plugin_dir);
     process_directory(plugin_dir, &head, process_plugin);
@@ -617,8 +621,7 @@ int process_plugins(void)
  * @param fd        Socket descriptor to lookup
  * @return struct plugin
  */
-struct plugin *plugin_lookup(int fd)
-{
+struct plugin *plugin_lookup(int fd) {
     struct plugin *plug = NULL;
     LIST_FOREACH(plug, &head, pointers) {
         if (plug->fd == fd) {
@@ -635,8 +638,7 @@ struct plugin *plugin_lookup(int fd)
  * @param require_root  int, indicate whether this plugin require root
  *                      privilege or not
  */
-void exec_plugin(char *plugin, int client_fd, int require_root)
-{
+void exec_plugin(char *plugin, int client_fd, int require_root) {
     int err = 0;
 
     info("Exec'ing plug-in = %s\n", plugin);
@@ -655,7 +657,7 @@ void exec_plugin(char *plugin, int client_fd, int require_root)
         /* Child */
         int exec_rc = 0;
         char fd_str[12];
-        char *plugin_argv[7];
+        const char *plugin_argv[7];
         extern char **environ;
         struct ucred cli_user_cred;
         socklen_t cli_user_cred_len = sizeof(cli_user_cred);
@@ -668,11 +670,13 @@ void exec_plugin(char *plugin, int client_fd, int require_root)
             drop_privileges();
         } else {
             if (getuid()) {
-                warn("Plugin %s require root privilege, but lsmd daemon "
-                     "is not run as root user\n", plugin);
+                warn("Plugin %s requires root privileges, but lsmd daemon "
+                     "is not running as root user\n",
+                     plugin);
             } else if (allow_root_plugin == 0) {
-                warn("Plugin %s require root privilege, but %s disabled "
-                     "it globally\n", LSMD_CONF_FILE);
+                warn("Plugin %s requires root privileges, but %s disables "
+                     "it globally\n",
+                     plugin, LSMD_CONF_FILE);
                 drop_privileges();
             } else {
                 /* Check socket client uid */
@@ -681,8 +685,9 @@ void exec_plugin(char *plugin, int client_fd, int require_root)
                                &cli_user_cred, &cli_user_cred_len);
                 if (0 == rc_get_cli_uid) {
                     if (cli_user_cred.uid != 0) {
-                        warn("Plugin %s require root privilege, but "
-                             "client is not run as root user\n", plugin);
+                        warn("Plugin %s requires root privileges, but "
+                             "client is not running as root user\n",
+                             plugin);
                         drop_privileges();
                     } else {
                         info("Plugin %s is running as root privilege\n",
@@ -690,11 +695,11 @@ void exec_plugin(char *plugin, int client_fd, int require_root)
                     }
                 } else {
                     warn("Failed to get client socket uid, getsockopt() "
-                         "error: %d\n", errno);
+                         "error: %d\n",
+                         errno);
                     drop_privileges();
                 }
             }
-
         }
 
         /* Make copy of plug-in string as once we call empty_plugin_list it
@@ -717,27 +722,31 @@ void exec_plugin(char *plugin, int client_fd, int require_root)
             plugin_argv[5] = fd_str;
             plugin_argv[6] = NULL;
 
-            exec_rc = execve("/usr/bin/valgrind", plugin_argv, environ);
+            exec_rc = execve("/usr/bin/valgrind", (char *const *)plugin_argv,
+                             environ);
         } else {
             plugin_argv[0] = basename(p_copy);
             plugin_argv[1] = fd_str;
             plugin_argv[2] = NULL;
-            exec_rc = execve(p_copy, plugin_argv, environ);
+            exec_rc = execve(p_copy, (char *const *)plugin_argv, environ);
         }
 
-        if (-1 == exec_rc) {
-            int err = errno;
-            log_and_exit("Error on exec'ing Plugin %s: %s\n",
-                         p_copy, strerror(err));
-        }
+        /*
+         * The only reason we would get here is if execve fails as execve
+         * does not return on success.
+         */
+        err = errno;
+        warn("Error on exec'ing Plugin: %s: %s (execve rc: %d)\n", p_copy,
+             strerror(err), exec_rc);
+        free(p_copy);
+        exit(1);
     }
 }
 
 /**
  * Main event loop
  */
-void _serving(void)
-{
+void _serving(void) {
     struct plugin *plug = NULL;
     struct timeval tmo;
     fd_set readfds;
@@ -795,8 +804,7 @@ void _serving(void)
 /**
  * Main entry for daemon to work
  */
-void serve(void)
-{
+void serve(void) {
     while (serve_state != EXIT) {
         if (serve_state == RESTART) {
             info("Reloading plug-ins\n");
@@ -807,21 +815,39 @@ void serve(void)
     clean_up();
 }
 
-int main(int argc, char *argv[])
-{
+int ipc_lock_file() {
+    static const char LFN[] = ".lsmd-ipc-lock";
+    char *lock_file = path_form(socket_dir, LFN);
+    int fd = open(lock_file, O_CREAT | O_CLOEXEC | O_RDWR, S_IRUSR | S_IWUSR);
+    free(lock_file);
+    if (fd != -1) {
+        int lock = flock(fd, LOCK_EX | LOCK_NB);
+        if (lock != 0) {
+            log_and_exit("An instance of lsmd is using %s socketdir\n",
+                         socket_dir);
+        }
+    } else {
+        int errno_cpy = errno;
+        log_and_exit("Unable to create socketdir lock file %s/%s reason: %s\n",
+                     socket_dir, LFN, strerror(errno_cpy));
+    }
+    return fd;
+}
+
+int main(int argc, char *argv[]) {
     int c = 0;
+    int lock_fd = -1;
 
     LIST_INIT(&head);
 
     /* Process command line arguments */
     while (1) {
         static struct option l_options[] = {
-            {"help", no_argument, 0, 'h'},  //Index 0
-            {"plugindir", required_argument, 0, 0}, //Index 1
-            {"socketdir", required_argument, 0, 0}, //Index 2
-            {"confdir", required_argument, 0, 0},   //Index 3
-            {0, 0, 0, 0}
-        };
+            {"help", no_argument, 0, 'h'},          // Index 0
+            {"plugindir", required_argument, 0, 0}, // Index 1
+            {"socketdir", required_argument, 0, 0}, // Index 2
+            {"confdir", required_argument, 0, 0},   // Index 3
+            {0, 0, 0, 0}};
 
         int option_index = 0;
         c = getopt_long(argc, argv, "hvd", l_options, &option_index);
@@ -847,7 +873,7 @@ int main(int argc, char *argv[])
 
         case 'h':
             usage();
-            break;
+            return EXIT_SUCCESS;
 
         case 'v':
             verbose_flag = 1;
@@ -882,7 +908,7 @@ int main(int argc, char *argv[])
 
     /* Check lsmd.conf */
     char *lsmd_conf_path = path_form(conf_dir, LSMD_CONF_FILE);
-    parse_conf_bool(lsmd_conf_path, LSM_CONF_ALLOW_ROOT_OPT_NAME,
+    parse_conf_bool(lsmd_conf_path, (char *)LSM_CONF_ALLOW_ROOT_OPT_NAME,
                     &allow_root_plugin);
     free(lsmd_conf_path);
 
@@ -897,14 +923,22 @@ int main(int argc, char *argv[])
     }
     flight_check();
 
+    /*
+     * Make sure there is only 1 instance of lsmd running against the same
+     * IPC sockets directory.
+     */
+    lock_fd = ipc_lock_file();
+
     /* Become a daemon if told we are not using systemd */
     if (!systemd) {
         if (-1 == daemon(0, 0)) {
             int err = errno;
+            close(lock_fd);
             log_and_exit("Error on calling daemon: %s\n", strerror(err));
         }
     }
 
     serve();
+    close(lock_fd);
     return EXIT_SUCCESS;
 }
