@@ -1,16 +1,6 @@
-# Copyright (C) 2015-2017 Red Hat, Inc.
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or any later version.
+# SPDX-License-Identifier: LGPL-2.1-or-later
 #
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; If not, see <http://www.gnu.org/licenses/>.
+# Copyright (C) 2015-2023 Red Hat, Inc.
 #
 # Author: Gris Ge <fge@redhat.com>
 
@@ -466,7 +456,7 @@ class MegaRAID(IPlugin):
                     ErrorNumber.PLUGIN_BUG,
                     "MegaRAID storcli failed with error %d: %s" %
                     (detail_status['ErrCd'], detail_status['ErrMsg']))
-            real_data = ctrl_output[0].get('Response Data')
+            real_data = ctrl_output[0].get('Response Data', {})
             if real_data and 'Response Data' in list(real_data.keys()):
                 return real_data['Response Data']
 
@@ -535,7 +525,8 @@ class MegaRAID(IPlugin):
             if "Capabilities" in ctrl_show_all_output and \
                     "Enable JBOD" in ctrl_show_all_output["Capabilities"]:
 
-                if ctrl_show_all_output["Capabilities"]["Enable JBOD"] == "Yes":
+                if ctrl_show_all_output["Capabilities"][
+                        "Enable JBOD"] == "Yes":
                     mode = System.MODE_HBA
                 else:
                     mode = System.MODE_HARDWARE_RAID
@@ -544,10 +535,11 @@ class MegaRAID(IPlugin):
                 # be executed first.
             elif "Status" in ctrl_show_all_output and \
                     "Current Personality" in ctrl_show_all_output["Status"]:
-                if ctrl_show_all_output["Status"]["Current Personality"] == "HBA-Mode ":
+                if ctrl_show_all_output["Status"][
+                        "Current Personality"] == "HBA-Mode ":
                     mode = System.MODE_HBA
                 else:
-                    mode = System.MODE_HBA
+                    mode = System.MODE_HARDWARE_RAID
             else:
                 mode = System.MODE_UNKNOWN
 
@@ -963,7 +955,7 @@ class MegaRAID(IPlugin):
             slot_nums.append(slot_num)
 
         # Handle request volume name, LSI only allow 15 characters.
-        name = re.sub('[^0-9a-zA-Z_\-]+', '', name)[:15]
+        name = re.sub('[^0-9a-zA-Z_-]+', '', name)[:15]
 
         if enclosure_str == ' ':
             drives_str = "drives=%s" % ','.join(slot_nums)
