@@ -1,18 +1,6 @@
+# SPDX-License-Identifier: LGPL-2.1-or-later
 #
-# Copyright (C) 2017 Red Hat, Inc.
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; If not, see <http://www.gnu.org/licenses/>.
+# Copyright (C) 2017-2023 Red Hat, Inc.
 #
 # Author: jumitche
 
@@ -74,7 +62,7 @@ class NFSPlugin(INfs, IStorageAreaNetwork):
 
     @staticmethod
     def _parse_options(optionstring):
-        """ Parse a comma separted option list into a dict """
+        """ Parse a comma separated option list into a dict """
         options = {}
 
         if optionstring is None:
