@@ -1,17 +1,7 @@
 /*
- * Copyright (C) 2016 Red Hat, Inc.
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (C) 2016-2023 Red Hat, Inc.
  *
  * Author: Gris Ge <fge@redhat.com>
  */
@@ -688,6 +678,15 @@ void _db_sql_trans_rollback(sqlite3 *db) {
                      NULL /* don't parse output */);
 }
 
+static void remove_trail_sep(int items_printed, char *s) {
+    int nul_pos = items_printed - strlen(", ") + 1;
+    if (nul_pos >= 0 && nul_pos < _BUFF_SIZE) {
+        s[nul_pos] = '\0';
+    } else {
+        s[0] = '\0';
+    }
+}
+
 int _db_data_add(char *err_msg, sqlite3 *db, const char *table_name, ...) {
     int rc = LSM_ERR_OK;
     char sql_cmd[_BUFF_SIZE * 4];
@@ -729,8 +728,8 @@ int _db_data_add(char *err_msg, sqlite3 *db, const char *table_name, ...) {
     va_end(arg);
 
     /* Remove the trailing ", " */
-    keys_str[keys_printed - strlen(", ") + 1] = '\0';
-    values_str[values_printed - strlen(", ") + 1] = '\0';
+    remove_trail_sep(keys_printed, keys_str);
+    remove_trail_sep(values_printed, values_str);
 
     _snprintf_buff(err_msg, rc, out, sql_cmd,
                    "INSERT INTO %s (%s) VALUES (%s);", table_name, keys_str,
