@@ -1,23 +1,13 @@
 /*
- * Copyright (C) 2016 Red Hat, Inc.
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (C) 2016-2023 Red Hat, Inc.
  *
  * Author: Gris Ge <fge@redhat.com>
  */
 
 #ifndef _SIMC_DB_TABLE_INIT_H_
-#define _SIMC_DB_TABLE_INIT_H
+#define _SIMC_DB_TABLE_INIT_H_
 
 #include "db.h"
 
