@@ -1,16 +1,6 @@
-# Copyright (C) 2011-2014 Red Hat, Inc.
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or any later version.
+# SPDX-License-Identifier: LGPL-2.1-or-later
 #
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; If not, see <http://www.gnu.org/licenses/>.
+# Copyright (C) 2011-2023 Red Hat, Inc.
 #
 # Author: Andy Grover <agrover at redhat com>
 #         Gris Ge <fge@redhat.com>
@@ -518,9 +508,10 @@ class TargetdStorage(IStorageAreaNetwork, INfs):
         """
         if tgt_masks is None:
             tgt_masks = self._tgt_masks()
-        return list(m for m in tgt_masks
-                    if (m['vol_name'] == vol_name and m['pool_name'] ==
-                        pool_name and m['ag_id'] == ag_id)) != []
+        return list(
+            m for m in tgt_masks
+            if (m['vol_name'] == vol_name and m['pool_name'] == pool_name
+                and m['ag_id'] == ag_id)) != []
 
     def _lsm_vol_of_id(self, vol_id, error=None):
         try:
