@@ -1,18 +1,8 @@
-# Copyright (C) 2011-2014 Red Hat, Inc.
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or any later version.
+# SPDX-License-Identifier: LGPL-2.1-or-later
 #
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
+# Copyright (C) 2011-2023 Red Hat, Inc.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; If not, see <http://www.gnu.org/licenses/>.
-#
-# Author: tasleson
+# Author: Tony Asleson <tasleson@redhat.com>
 
 import hashlib
 
@@ -500,6 +490,7 @@ class ErrorNumber(object):
     NOT_FOUND_FS_SS = 204
     NOT_FOUND_VOLUME = 205
     NOT_FOUND_NFS_EXPORT = 206
+    NOT_FOUND_GENERIC = 207
     NOT_FOUND_SYSTEM = 208
     NOT_FOUND_DISK = 209
 
