@@ -1,19 +1,9 @@
-# Copyright (C) 2011-2016 Red Hat, Inc.
+# SPDX-License-Identifier: LGPL-2.1-or-later
+#
+# Copyright (C) 2011-2023 Red Hat, Inc.
 # (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or any later version.
 #
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; If not, see <http://www.gnu.org/licenses/>.
-#
-# Author: tasleson
+# Author: Tony Asleson <tasleson@redhat.com>
 #         Gris Ge <fge@redhat.com>
 #         Joe Handzik <joseph.t.handzik@hpe.com>
 
@@ -878,7 +868,7 @@ class AccessGroup(IData):
            10:00:00:00:c9:95:2f:de
 
         Acceptable WWPN format is:
-            ^[0x|0X]{0,1}(:?[0-9A-Fa-f]{2}[\.\-:]{0,1}){7}[0-9A-Fa-f]{2}$
+            ^[0x|0X]{0,1}(:?[0-9A-Fa-f]{2}[.-:]{0,1}){7}[0-9A-Fa-f]{2}$
         Acceptable WWPN example:
            10:00:00:00:c9:95:2f:de
            10:00:00:00:C9:95:2F:DE
