@@ -1,19 +1,9 @@
-# Copyright (C) 2012-2016 Red Hat, Inc.
+# SPDX-License-Identifier: LGPL-2.1-or-later
+#
+# Copyright (C) 2012-2024 Red Hat, Inc.
 # (C) Copyright 2017 Hewlett Packard Enterprise Development LP
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or any later version.
 #
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; If not, see <http://www.gnu.org/licenses/>.
-#
-# Author: tasleson
+# Author: Tony Asleson <tasleson@redhat.com>
 #         Gris Ge <fge@redhat.com>
 import argparse
 import os
@@ -39,11 +29,12 @@ from lsm.lsmcli.data_display import (DisplayData, PlugData, out,
                                      vol_provision_str_to_type,
                                      vol_rep_type_str_to_type, VolumeRAIDInfo,
                                      PoolRAIDInfo, VcrCap, LocalDiskInfo,
-                                     VolumeRAMCacheInfo)
+                                     VolumeRAMCacheInfo, LocalLedSlotInfo)
 
 _CONNECTION_FREE_COMMANDS = [
     'local-disk-list', 'local-disk-ident-led-on', 'local-disk-ident-led-off',
-    'local-disk-fault-led-on', 'local-disk-fault-led-off'
+    'local-disk-fault-led-on', 'local-disk-fault-led-off',
+    'local-led-slot-list', 'local-led-slot-set'
 ]
 
 if six.PY3:
@@ -247,6 +238,33 @@ def _valid_ip6_address(address):
     return False
 
 
+def _check_led_state(state):
+    # Make sure the state is in the correct form
+    valid_options = dict(IDENT_ON=Disk.LED_STATUS_IDENT_ON,
+                         IDENT_OFF=Disk.LED_STATUS_IDENT_OFF,
+                         FAULT_ON=Disk.LED_STATUS_FAULT_ON,
+                         FAULT_OFF=Disk.LED_STATUS_FAULT_OFF)
+    t_states = state.upper().split(":")
+    for s in t_states:
+        if s not in valid_options.keys():
+            raise ArgumentTypeError("Invalid state '%s', valid choices = %s" %
+                                    (s, list(valid_options.keys())))
+
+    if len(t_states) == len(set(t_states)):
+        if len(t_states) > 2:
+            raise ArgumentTypeError("Maximum number of LED state values is 2!")
+        if len(t_states) == 2:
+            if t_states[0][0:4] == t_states[1][0:4]:
+                raise ArgumentTypeError(
+                    "States which counteract one another are invalid!")
+            return valid_options[t_states[0]] | valid_options[t_states[1]]
+        else:
+            return valid_options[t_states[0]]
+    else:
+        raise ArgumentTypeError("Repeated same LED state is invalid! %s" %
+                                t_states)
+
+
 def _is_valid_network_name(ip_hn):
     """
     Checks to see if the supplied string is a valid ip4/6 or hostname
@@ -632,6 +650,17 @@ local_disk_path_opt = dict(name='--path',
                            help="Local disk path",
                            metavar='<DISK_PATH>')
 
+local_disk_led_slot_opt = dict(name='--slot',
+                               help="LED slot identifier",
+                               metavar='<slot_id>')
+
+local_disk_led_slot_state_opt = dict(
+    name="--state",
+    help=
+    "LED slot value state (IDENT_[ON|OFF] or [FAULT_[ON|OFF], or combination separated with ':')",
+    metavar='<state>',
+    type=_check_led_state)
+
 cmds = (
     dict(
         name='list',
@@ -1147,6 +1176,12 @@ cmds = (
         args=[],
         optional=[],
     ),
+    dict(
+        name='local-led-slot-list',
+        help='Query slots with LED identification',
+        args=[],
+        optional=[],
+    ),
     dict(
         name='volume-cache-info',
         help='Query volume RAM cache information',
@@ -1206,6 +1241,14 @@ cmds = (
             dict(local_disk_path_opt),
         ],
     ),
+    dict(
+        name='local-led-slot-set',
+        help='Sets the state of the specified LED',
+        args=[
+            dict(local_disk_led_slot_opt),
+            dict(local_disk_led_slot_state_opt),
+        ],
+    ),
 )
 
 aliases = dict(
@@ -2236,6 +2279,16 @@ class CmdLine(object):
 
         self.display_data(local_disks)
 
+    def local_led_slot_list(self, args):
+        local_led_slots = []
+
+        with LocalDisk.led_slots_open() as slots:
+            for s in slots:
+                local_led_slots.append(
+                    LocalLedSlotInfo(s.id(), s.state(), s.device()))
+
+        self.display_data(local_led_slots)
+
     def volume_cache_info(self, args):
         lsm_vol = _get_item(self.c.volumes(), args.vol, "Volume")
         self.display_data([
@@ -2288,3 +2341,12 @@ class CmdLine(object):
 
     def local_disk_fault_led_off(self, args):
         LocalDisk.fault_led_off(args.path)
+
+    def local_led_slot_set(self, args):
+        with LocalDisk.led_slots_open() as slots:
+            for s in slots:
+                if s.id() == args.slot:
+                    s.state_set(args.state)
+                    return
+
+        raise ArgError("LED slot id '%s' not found!" % (args.slot))
