@@ -1,19 +1,10 @@
-# Copyright (C) 2014 Red Hat, Inc.
+# SPDX-License-Identifier: LGPL-2.1-or-later
+#
+# Copyright (C) 2014-2023 Red Hat, Inc.
 # (C) Copyright 2017 Hewlett Packard Enterprise Development LP
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; If not, see <http://www.gnu.org/licenses/>.
 #
 # Author: Gris Ge <fge@redhat.com>
+
 import sys
 from datetime import datetime
 
@@ -435,6 +426,14 @@ class LocalDiskInfo(object):
         self.health_status = health_status
 
 
+class LocalLedSlotInfo(object):
+
+    def __init__(self, id, state, device):
+        self.id = id
+        self.state = state
+        self.device = device
+
+
 class VolumeRAMCacheInfo(object):
 
     _PHY_DISK_CACHE_STATUS_MAP = {
@@ -860,6 +859,22 @@ class DisplayData(object):
         'value_conv_human': LOCAL_DISK_VALUE_CONV_HUMAN,
     }
 
+    LOCAL_LED_SLOT_HEADER = OrderedDict()
+    LOCAL_LED_SLOT_HEADER['id'] = 'Slot ID'
+    LOCAL_LED_SLOT_HEADER['state'] = 'LED state'
+    LOCAL_LED_SLOT_HEADER['device'] = 'Device Node'
+
+    LOCAL_LED_SLOT_SKIP_KEYS = []
+    LOCAL_LED_SLOT_CONV_ENUM = {'state': disk_led_status_to_str}
+    LOCAL_LED_SLOT_CONV_HUMAN = []
+
+    VALUE_CONVERT[LocalLedSlotInfo] = {
+        'headers': LOCAL_LED_SLOT_HEADER,
+        'column_skip_keys': LOCAL_LED_SLOT_SKIP_KEYS,
+        'value_conv_enum': LOCAL_LED_SLOT_CONV_ENUM,
+        'value_conv_human': LOCAL_LED_SLOT_CONV_HUMAN,
+    }
+
     BATTERY_HEADER = OrderedDict()
     BATTERY_HEADER['id'] = 'ID'
     BATTERY_HEADER['name'] = 'Name'
